@@ -36,6 +36,8 @@ STDAPI PrxDllUnregisterServer(void);
 #endif
 
 //{{AFX_INSERT_LOCATION}}
-// Microsoft Visual C++ will insert additional declarations immediately before the previous line.
+// Microsoft Visual C++ will insert additional declarations
+// immediately before the previous line.
 
-#endif // !defined(AFX_DLLDATAX_H__281C69AC_6B7B_4FA8_9B5A_84546EFB391A__INCLUDED_)
+#endif 
+// !defined(AFX_DLLDATAX_H__281C69AC_6B7B_4FA8_9B5A_84546EFB391A__INCLUDED_)
