@@ -26,6 +26,7 @@
 #include <apr_strings.h>
 #include <apr_md5.h>
 #include <apr_portable.h>
+#include <apr_xml.h>
 
 #include <ne_socket.h>
 #include <ne_basic.h>
@@ -49,6 +50,8 @@
 #include "svn_dav.h"
 #include "svn_time.h"
 
+#include "svn_private_config.h"
+
 #include "ra_dav.h"
 
 
@@ -251,7 +254,7 @@ static svn_error_t *simple_store_vsn_url(const char *vsn_url,
   /* store the version URL as a property */
   SVN_ERR_W( (*setter)(baton, SVN_RA_DAV__LP_VSN_URL, 
                        svn_string_create(vsn_url, pool), pool),
-             "Could not save the URL of the version resource" );
+             _("Could not save the URL of the version resource") );
 
   return NULL;
 }
@@ -395,7 +398,7 @@ static svn_error_t *custom_get_request(ne_session *sess,
   if (req == NULL)
     {
       return svn_error_createf(SVN_ERR_RA_DAV_CREATING_REQUEST, NULL,
-                               "Could not create a GET request for '%s'",
+                               _("Could not create a GET request for '%s'"),
                                url);
     }
 
@@ -459,10 +462,10 @@ static svn_error_t *custom_get_request(ne_session *sess,
     {
        const char *msg;
 
-       msg = apr_psprintf(pool, "GET request failed for %s", url);
+       msg = apr_psprintf(pool, _("GET request failed for '%s'"), url);
        if (err)
          svn_error_clear (err);
-       err = svn_ra_dav__convert_error(sess, msg, decompress_rv);
+       err = svn_ra_dav__convert_error(sess, msg, decompress_rv, pool);
     }
   
   if (err)
@@ -582,7 +585,7 @@ static svn_error_t *simple_fetch_file(ne_session *sess,
                                         pool,
                                         &frc.handler,
                                         &frc.handler_baton),
-             "Could not save file");
+             _("Could not save file"));
 
   /* Only bother with text-deltas if our caller cares. */
   if (! text_deltas)
@@ -619,18 +622,19 @@ static void get_file_reader(void *userdata, const char *buf, size_t len)
 
   /* Write however many bytes were passed in by neon. */
   wlen = len;
-  svn_stream_write(stream, buf, &wlen);
+  svn_error_clear(svn_stream_write(stream, buf, &wlen));
 
 #if 0
   /* Neon's callback won't let us return error.  Joe knows this is a
      bug in his API, so this section can be reactivated someday. */
 
+  SVN_ERR(svn_stream_write(stream, buf, &wlen));
   if (wlen != len)
     {
       /* Uh oh, didn't write as many bytes as neon gave us. */
       return 
         svn_error_create(SVN_ERR_STREAM_UNEXPECTED_EOF, NULL,
-                         "Error writing to stream: unexpected EOF");
+                         _("Error writing to stream: unexpected EOF"));
     }
 #endif
       
@@ -815,9 +819,9 @@ svn_error_t *svn_ra_dav__get_file(void *session_baton,
           if (strcmp (hex_digest, expected_checksum->data) != 0)
             return svn_error_createf
               (SVN_ERR_CHECKSUM_MISMATCH, NULL,
-               "Checksum mismatch for '%s':\n"
-               "   expected checksum:  %s\n"
-               "   actual checksum:    %s\n",
+               _("Checksum mismatch for '%s':\n"
+                 "   expected checksum:  %s\n"
+                 "   actual checksum:    %s\n"),
                path, expected_checksum->data, hex_digest);
         }
     }
@@ -845,10 +849,9 @@ svn_error_t *svn_ra_dav__get_dir(void *session_baton,
 {
   svn_ra_dav_resource_t *rsrc;
   apr_hash_index_t *hi;
-  int len;
   apr_hash_t *resources;
   const char *final_url;
-  char *stripped_final_url;
+  apr_size_t final_url_n_components;
   svn_ra_session_t *ras = (svn_ra_session_t *) session_baton;
   const char *url = svn_path_url_add_component (ras->url, path, pool);
 
@@ -884,13 +887,9 @@ svn_error_t *svn_ra_dav__get_dir(void *session_baton,
                                      final_url, NE_DEPTH_ONE,
                                      NULL, NULL /* all props */, pool) );
       
-      /* Clean up any trailing slashes on final_url, creating
-         stripped_final_url */
-      stripped_final_url = apr_pstrdup(pool, final_url);
-      len = strlen(final_url);
-      if (len > 1 && final_url[len - 1] == '/')
-        stripped_final_url[len - 1] = '\0';
-      
+      /* Count the number of path components in final_url. */
+      final_url_n_components = svn_path_component_count(final_url);
+
       /* Now we have a hash that maps a bunch of url children to resource
          objects.  Each resource object contains the properties of the
          child.   Parse these resources into svn_dirent_t structs. */
@@ -911,8 +910,11 @@ svn_error_t *svn_ra_dav__get_dir(void *session_baton,
           childname =  key;
           resource = val;
           
-          /* Skip the effective '.' entry that comes back from NE_DEPTH_ONE */
-          if (strcmp(resource->url, stripped_final_url) == 0)
+          /* Skip the effective '.' entry that comes back from NE_DEPTH_ONE.
+             The children must have one more component then final_url.
+             Note that we can't just strcmp the URLs because of URL encoding
+             differences (i.e. %3c vs. %3C etc.) */
+          if (svn_path_component_count(childname) == final_url_n_components)
             continue;
           
           entry = apr_pcalloc (pool, sizeof(*entry));
@@ -1058,8 +1060,14 @@ svn_error_t *svn_ra_dav__get_dated_revision (void *session_baton,
 {
   svn_ra_session_t *ras = session_baton;
   const char *body;
+  const char *vcc_url;
   svn_error_t *err;
 
+  /* Run the 'dated-rev-report' on the VCC url, which is always
+     guaranteed to exist.   */
+  SVN_ERR (svn_ra_dav__get_vcc(&vcc_url, ras->sess, ras->root.path, pool));
+  
+
   body = apr_psprintf(pool,
                       "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
                       "<S:dated-rev-report xmlns:S=\"" SVN_XML_NAMESPACE "\" "
@@ -1070,41 +1078,163 @@ svn_error_t *svn_ra_dav__get_dated_revision (void *session_baton,
 
   *revision = SVN_INVALID_REVNUM;
   err = svn_ra_dav__parsed_request_compat(ras->sess, "REPORT",
-                                          ras->root.path, body, NULL, NULL,
+                                          vcc_url, body, NULL, NULL,
                                           drev_report_elements,
                                           drev_validate_element,
                                           drev_start_element, drev_end_element,
                                           revision, NULL, NULL, pool);
   if (err && err->apr_err == SVN_ERR_UNSUPPORTED_FEATURE)
-    return svn_error_quick_wrap(err, "Server does not support date-based "
-                                "operations");
+    return svn_error_quick_wrap(err, _("Server does not support date-based "
+                                       "operations"));
   else if (err)
     return err;
 
   if (*revision == SVN_INVALID_REVNUM)
     return svn_error_create(SVN_ERR_INCOMPLETE_DATA, NULL,
-                            "Invalid server response to dated-rev request");
+                            _("Invalid server response to dated-rev request"));
 
   return SVN_NO_ERROR;
 }
 
+typedef struct {
+  svn_ra_session_t *ras;
+  apr_hash_t *hash;
+  apr_pool_t *pool;
+} get_locations_baton_t;
 
-/* Populate the members of ne_propname structure *PROP for the
-   Subversion property NAME.  */
-static void
-make_ne_propname (ne_propname *prop,
-                  const char *name)
+/*
+ * Plan for processing the XML. The XML will be of the form:
+ *
+ * <S:get-locations-report xmlns...>
+ *     <S:location rev="..." path="..."/>
+ *     ...
+ * </S:get-locations-report>
+ *
+ * We extract what we want at the start of <S:location>. */
+/* Elements used in a get-locations response */
+static const svn_ra_dav__xml_elm_t gloc_report_elements[] =
 {
-  if (strncmp(name, SVN_PROP_PREFIX, (sizeof(SVN_PROP_PREFIX) - 1)) == 0)
+  { SVN_XML_NAMESPACE, "get-locations-report", ELEM_get_locations_report, 0 },
+  { SVN_XML_NAMESPACE, "location", ELEM_location, 0 },
+  { NULL }
+};
+
+static const char *get_attr(const char **atts, const char *which);
+
+/* This implements the `ne_xml_startelem_cb' prototype. */
+static int gloc_start_element(void *userdata, int parent_state, const char *ns,
+                              const char *ln, const char **atts)
+{
+  get_locations_baton_t *baton = userdata;
+  const svn_ra_dav__xml_elm_t *elm;
+
+  elm = svn_ra_dav__lookup_xml_elem(gloc_report_elements, ns, ln);
+
+  /* Just skip unknown elements. */
+  if (!elm)
+    return NE_XML_DECLINE;
+
+  if (parent_state == ELEM_get_locations_report
+      && elm->id == ELEM_location)
     {
-      prop->nspace = SVN_DAV_PROP_NS_SVN;
-      prop->name = name + sizeof(SVN_PROP_PREFIX) - 1;
+      svn_revnum_t rev = SVN_INVALID_REVNUM;
+      const char *path;
+      const char *r;
+
+      r = get_attr(atts, "rev");
+      if (r)
+        rev = SVN_STR_TO_REV(r);
+
+      path = get_attr(atts, "path");
+
+      if (SVN_IS_VALID_REVNUM(rev) && path)
+        apr_hash_set(baton->hash,
+                     apr_pmemdup(baton->pool, &rev, sizeof (rev)),
+                     sizeof(rev), apr_pstrdup(baton->pool, path));
+      else
+        return NE_XML_ABORT;
     }
-  else
+
+  return elm->id;
+}
+
+svn_error_t *
+svn_ra_dav__get_locations(void *session_baton,
+                          apr_hash_t **locations,
+                          const char *relative_path,
+                          svn_revnum_t peg_revision,
+                          apr_array_header_t *location_revisions,
+                          apr_pool_t *pool)
+{
+  svn_ra_session_t *ras = session_baton;
+  svn_stringbuf_t *request_body;
+  svn_error_t *err;
+  get_locations_baton_t request_baton;
+  const char *relative_path_quoted;
+  svn_string_t bc_url, bc_relative;
+  const char *final_bc_url;
+  int i;
+  int status_code = 0;
+
+  *locations = apr_hash_make (pool);
+
+  request_body = svn_stringbuf_create("", pool);
+  svn_stringbuf_appendcstr(request_body,
+                           "<?xml version=\"1.0\" encoding=\"utf-8\"?>" DEBUG_CR
+                           "<S:get-locations xmlns:S=\"" SVN_XML_NAMESPACE
+                           "\" xmlns:D=\"DAV:\">" DEBUG_CR);
+
+  svn_stringbuf_appendcstr(request_body, "<S:path>");
+  /* We need to escape the path XML-wise. */
+  relative_path_quoted = apr_xml_quote_string(pool, relative_path, 0);
+  svn_stringbuf_appendcstr(request_body, relative_path_quoted);
+  svn_stringbuf_appendcstr(request_body, "</S:path>" DEBUG_CR);
+  svn_stringbuf_appendcstr(request_body,
+                           apr_psprintf(pool,
+                                        "<S:peg-revision>%ld"
+                                        "</S:peg-revision>" DEBUG_CR,
+                                        peg_revision));
+
+  for (i = 0; i < location_revisions->nelts; ++i)
     {
-      prop->nspace = SVN_DAV_PROP_NS_CUSTOM;
-      prop->name = name;
+      svn_revnum_t rev = APR_ARRAY_IDX(location_revisions, i, svn_revnum_t);
+      svn_stringbuf_appendcstr(request_body,
+                               apr_psprintf(pool,
+                                            "<S:location-revision>%ld"
+                                            "</S:location-revision>" DEBUG_CR,
+                                            rev));
     }
+
+  svn_stringbuf_appendcstr (request_body, "</S:get-locations>");
+
+  request_baton.ras = ras;
+  request_baton.hash = *locations;
+  request_baton.pool = pool;
+
+  /* ras's URL may not exist in HEAD, and thus it's not safe to send
+     it as the main argument to the REPORT request; it might cause
+     dav_get_resource() to choke on the server.  So instead, we pass a
+     baseline-collection URL, which we get from the largest of the
+     START and END revisions. */
+  SVN_ERR( svn_ra_dav__get_baseline_info(NULL, &bc_url, &bc_relative, NULL,
+                                         ras->sess, ras->url, peg_revision,
+                                         ras->pool) );
+  final_bc_url = svn_path_url_add_component(bc_url.data, bc_relative.data,
+                                            ras->pool);
+
+  err = svn_ra_dav__parsed_request(ras->sess, "REPORT", final_bc_url,
+                                   request_body->data, NULL, NULL,
+                                   gloc_start_element, NULL, NULL,
+                                   &request_baton, NULL, &status_code,
+                                   pool);
+
+  /* Map status 501: Method Not Implemented to our not implemented error.
+     1.0.x servers and older don't support this report. */
+  if (status_code == 501)
+    return svn_error_create (SVN_ERR_RA_NOT_IMPLEMENTED, err,
+                             _("get-locations REPORT not implemented"));
+
+  return err;
 }
 
 svn_error_t *svn_ra_dav__change_rev_prop (void *session_baton,
@@ -1170,8 +1300,8 @@ svn_error_t *svn_ra_dav__change_rev_prop (void *session_baton,
     return 
       svn_error_create
       (SVN_ERR_RA_DAV_REQUEST_FAILED, err,
-       "DAV request failed; it's possible that the repository's "
-       "pre-revprop-change hook either failed or is non-existent");
+       _("DAV request failed; it's possible that the repository's "
+         "pre-revprop-change hook either failed or is non-existent"));
 
   return SVN_NO_ERROR;
 }
@@ -1211,36 +1341,16 @@ svn_error_t *svn_ra_dav__rev_prop (void *session_baton,
                                    svn_string_t **value,
                                    apr_pool_t *pool)
 {
-  svn_ra_session_t *ras = session_baton;
-  svn_ra_dav_resource_t *baseline;
-  apr_hash_t *filtered_props;
+  apr_hash_t *props;
 
-  /* E-Z initialization */
-  ne_propname wanted_props[] =
-    {
-      { "", "" },
-      { NULL }
-    };
+  /* We just call svn_ra_dav__rev_proplist() and filter its results here
+   * because sending the property name to the server may create an error
+   * if it has a colon in its name.  While more costly this allows DAV
+   * clients to still gain access to all the allowed property names.
+   * See Issue #1807 for more details. */
+  SVN_ERR (svn_ra_dav__rev_proplist (session_baton, rev, &props, pool));
 
-  /* Decide on the namespace and propname for XML marshalling. */
-  make_ne_propname(&(wanted_props[0]), name);
-
-  /* Main objective: do a PROPFIND (allprops) on a baseline object */  
-  SVN_ERR (svn_ra_dav__get_baseline_props(NULL, &baseline,
-                                          ras->sess, 
-                                          ras->url,
-                                          rev,
-                                          wanted_props,
-                                          pool));
-
-  /* Build a new property hash, based on the one in the baseline
-     resource.  In particular, convert the xml-property-namespaces
-     into ones that the client understands.  Strip away the DAV:
-     liveprops as well. */
-  filtered_props = apr_hash_make(pool);
-  SVN_ERR (filter_props (filtered_props, baseline, FALSE, pool));
-
-  *value = apr_hash_get(filtered_props, name, APR_HASH_KEY_STRING);
+  *value = apr_hash_get (props, name, APR_HASH_KEY_STRING);
 
   return SVN_NO_ERROR;
 }
@@ -1480,7 +1590,7 @@ start_element(void *userdata, int parent_state, const char *nspace,
     case ELEM_resource:
       att = get_attr(atts, "path");
       /* ### verify we got it. punt on error. */
-      rb->current_wcprop_path = apr_pstrdup(rb->ras->pool, att);
+      rb->current_wcprop_path = svn_path_join(rb->target, att, rb->ras->pool);
       break;
 
     case ELEM_open_directory:
@@ -1911,9 +2021,10 @@ static int cdata_handler(void *userdata, int state,
         if (nlen != len)
           {
             /* Short write without associated error?  "Can't happen." */
-            CHKERR( svn_error_createf(SVN_ERR_STREAM_UNEXPECTED_EOF, NULL,
-                                      "Error writing to '%s': unexpected EOF",
-                                      rb->namestr->data) );
+            CHKERR( svn_error_createf
+                    (SVN_ERR_STREAM_UNEXPECTED_EOF, NULL,
+                     _("Error writing to '%s': unexpected EOF"),
+                     rb->namestr->data) );
           }
       }
       break;
@@ -2049,7 +2160,7 @@ static int end_element(void *userdata, int state,
         else
           {
             CHKERR( svn_error_createf(SVN_ERR_XML_UNKNOWN_ENCODING, NULL,
-                                      "Unknown XML encoding: '%s'",
+                                      _("Unknown XML encoding: '%s'"),
                                       rb->encoding->data) );
             abort(); /* Not reached. */
           }
@@ -2174,15 +2285,13 @@ static svn_error_t * reporter_set_path(void *report_baton,
   svn_xml_escape_cdata_cstring (&qpath, path, pool);
   if (start_empty)
     entry = apr_psprintf(pool,
-                         "<S:entry rev=\"%"
-                         SVN_REVNUM_T_FMT
-                         "\" start-empty=\"true\">%s</S:entry>" DEBUG_CR,
+                         "<S:entry rev=\"%ld\""
+                         " start-empty=\"true\">%s</S:entry>" DEBUG_CR,
                          revision, qpath->data);
   else
     entry = apr_psprintf(pool,
-                         "<S:entry rev=\"%"
-                         SVN_REVNUM_T_FMT
-                         "\">%s</S:entry>" DEBUG_CR,
+                         "<S:entry rev=\"%ld\">"
+                         "%s</S:entry>" DEBUG_CR,
                          revision, qpath->data);
 
   return svn_io_file_write_full(rb->tmpfile, entry, strlen(entry), NULL, pool);
@@ -2214,14 +2323,14 @@ static svn_error_t * reporter_link_path(void *report_baton,
   svn_xml_escape_attr_cstring (&qlinkpath, bc_relative.data, pool);
   if (start_empty)
     entry = apr_psprintf(pool,
-                         "<S:entry rev=\"%" SVN_REVNUM_T_FMT
-                         "\" linkpath=\"/%s\" start-empty=\"true\""
+                         "<S:entry rev=\"%ld\""
+                         " linkpath=\"/%s\" start-empty=\"true\""
                          ">%s</S:entry>" DEBUG_CR,
                          revision, qlinkpath->data, qpath->data);
   else
     entry = apr_psprintf(pool,
-                         "<S:entry rev=\"%" SVN_REVNUM_T_FMT
-                         "\" linkpath=\"/%s\">%s</S:entry>" DEBUG_CR,
+                         "<S:entry rev=\"%ld\""
+                         " linkpath=\"/%s\">%s</S:entry>" DEBUG_CR,
                          revision, qlinkpath->data, qpath->data);
 
   return svn_io_file_write_full(rb->tmpfile, entry, strlen(entry), NULL, pool);
@@ -2309,7 +2418,7 @@ static svn_error_t * reporter_finish_report(void *report_baton,
     {
       return svn_error_createf 
         (SVN_ERR_RA_DAV_REQUEST_FAILED, NULL,
-         "REPORT response handling failed to complete the editor drive");
+         _("REPORT response handling failed to complete the editor drive"));
     }
 
   /* store auth info if we can. */
@@ -2399,7 +2508,7 @@ make_reporter (void *session_baton,
   if (SVN_IS_VALID_REVNUM(revision))
     {
       s = apr_psprintf(pool, 
-                       "<S:target-revision>%" SVN_REVNUM_T_FMT
+                       "<S:target-revision>%ld"
                        "</S:target-revision>", revision);
       SVN_ERR( svn_io_file_write_full(rb->tmpfile, s, strlen(s), NULL, pool) );
     }
