@@ -16,6 +16,7 @@
  */
 
 
+#include "svn_private_config.h"
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_fs.h"
@@ -41,6 +42,8 @@ struct parse_baton
   svn_stream_t *outstream;
   enum svn_repos_load_uuid uuid_action;
   const char *parent_dir;
+  apr_pool_t *pool;
+  apr_hash_t *rev_map;
 };
 
 struct revision_baton
@@ -74,6 +77,31 @@ struct node_baton
 
 
 
+/*----------------------------------------------------------------------*/
+
+/** A conversion function between the two vtable types. **/
+static
+svn_repos_parser_fns2_t *fns2_from_fns (const svn_repos_parser_fns_t *fns,
+                                        apr_pool_t *pool)
+{
+  svn_repos_parser_fns2_t *fns2;
+
+  fns2 = apr_palloc (pool, sizeof (*fns2));
+  fns2->new_revision_record = fns->new_revision_record;
+  fns2->uuid_record = fns->uuid_record;
+  fns2->new_node_record = fns->new_node_record;
+  fns2->set_revision_property = fns->set_revision_property;
+  fns2->set_node_property = fns->set_node_property;
+  fns2->remove_node_props = fns->remove_node_props;
+  fns2->set_fulltext = fns->set_fulltext;
+  fns2->close_node = fns->close_node;
+  fns2->close_revision = fns->close_revision;
+  fns2->delete_node_property = NULL;
+  fns2->apply_textdelta = NULL;
+  return fns2;
+}
+
+
 /*----------------------------------------------------------------------*/
 
 /** The parser and related helper funcs **/
@@ -172,7 +200,7 @@ stream_malformed (void)
 static svn_error_t *
 parse_property_block (svn_stream_t *stream,
                       svn_filesize_t content_length,
-                      const svn_repos_parser_fns_t *parse_fns,
+                      const svn_repos_parser_fns2_t *parse_fns,
                       void *record_baton,
                       svn_boolean_t is_node,
                       apr_pool_t *pool)
@@ -276,6 +304,41 @@ parse_property_block (svn_stream_t *stream,
           else
             return stream_malformed (); /* didn't find expected 'V' line */
         }
+      else if ((buf[0] == 'D') && (buf[1] == ' '))
+        {
+          apr_size_t numread;
+          char *keybuf;
+          char c;
+          
+          /* Get the length of the key */
+          apr_size_t keylen = (apr_size_t) atoi (buf + 2);
+
+          /* Now read that much into a buffer, + 1 byte for null terminator */
+          keybuf = apr_pcalloc (pool, keylen + 1);
+          numread = keylen;
+          SVN_ERR (svn_stream_read (stream, keybuf, &numread));
+          content_length -= numread;
+          if (numread != keylen)
+            return stream_ran_dry ();
+          keybuf[keylen] = '\0';
+
+          /* Suck up extra newline after key data */
+          numread = 1;
+          SVN_ERR (svn_stream_read (stream, &c, &numread));
+          content_length -= numread;
+          if (numread != 1)
+            return stream_ran_dry ();
+          if (c != '\n') 
+            return stream_malformed ();
+
+          /* We don't expect these in revision properties, and if we see
+             one when we don't have a delete_node_property callback,
+             then we're seeing a v3 feature in a v2 dump. */
+          if (!is_node || !parse_fns->delete_node_property)
+            return stream_malformed ();
+
+          SVN_ERR (parse_fns->delete_node_property (record_baton, keybuf));
+        }
       else
         return stream_malformed (); /* didn't find expected 'K' line */
       
@@ -293,7 +356,8 @@ parse_property_block (svn_stream_t *stream,
 static svn_error_t *
 parse_text_block (svn_stream_t *stream,
                   svn_filesize_t content_length,
-                  const svn_repos_parser_fns_t *parse_fns,
+                  svn_boolean_t is_delta,
+                  const svn_repos_parser_fns2_t *parse_fns,
                   void *record_baton,
                   char *buffer,
                   apr_size_t buflen,
@@ -301,9 +365,21 @@ parse_text_block (svn_stream_t *stream,
 {
   svn_stream_t *text_stream = NULL;
   apr_size_t num_to_read, rlen, wlen;
-  
-  /* Get a stream to which we can push the data. */
-  SVN_ERR (parse_fns->set_fulltext (&text_stream, record_baton));
+
+  if (is_delta)
+    {
+      svn_txdelta_window_handler_t wh;
+      void *whb;
+
+      SVN_ERR (parse_fns->apply_textdelta (&wh, &whb, record_baton));
+      if (wh)
+        text_stream = svn_txdelta_parse_svndiff (wh, whb, TRUE, pool);
+    }
+  else
+    {
+      /* Get a stream to which we can push the data. */
+      SVN_ERR (parse_fns->set_fulltext (&text_stream, record_baton));
+    }
 
   /* If there are no contents to read, just write an empty buffer
      through our callback. */
@@ -384,12 +460,12 @@ parse_format_version (const char *versionstring, int *version)
 
 /* The Main Parser Logic */
 svn_error_t *
-svn_repos_parse_dumpstream (svn_stream_t *stream,
-                            const svn_repos_parser_fns_t *parse_fns,
-                            void *parse_baton,
-                            svn_cancel_func_t cancel_func,
-                            void *cancel_baton,
-                            apr_pool_t *pool)
+svn_repos_parse_dumpstream2 (svn_stream_t *stream,
+                             const svn_repos_parser_fns2_t *parse_fns,
+                             void *parse_baton,
+                             svn_cancel_func_t cancel_func,
+                             void *cancel_baton,
+                             apr_pool_t *pool)
 {
   svn_boolean_t eof;
   svn_stringbuf_t *linebuf;
@@ -409,6 +485,14 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
      number, and a blank line. */
   SVN_ERR (parse_format_version (linebuf->data, &version));
 
+  /* If we were called from svn_repos_parse_dumpstream(), the
+     callbacks to handle delta contents will be NULL, so we have to
+     reject dumpfiles with the current version. */
+  if (version == SVN_REPOS_DUMPFILE_FORMAT_VERSION
+      && (!parse_fns->delete_node_property || !parse_fns->apply_textdelta))
+    return svn_error_createf (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
+                              "Unsupported dumpfile version: %d", version);
+
   /* A dumpfile "record" is defined to be a header-block of
      rfc822-style headers, possibly followed by a content-block.
 
@@ -509,8 +593,14 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
                                   SVN_REPOS_DUMPFILE_PROP_CONTENT_LENGTH,
                                   APR_HASH_KEY_STRING)))
         {
-          /* First, remove all node properties. */
-          if (found_node)
+          const char *delta = apr_hash_get (headers,
+                                            SVN_REPOS_DUMPFILE_PROP_DELTA,
+                                            APR_HASH_KEY_STRING);
+          svn_boolean_t is_delta = (delta && strcmp (delta, "true") == 0);
+
+          /* First, remove all node properties, unless this is a delta
+             property block. */
+          if (found_node && !is_delta)
             SVN_ERR (parse_fns->remove_node_props (node_baton));
 
           SVN_ERR (parse_property_block (stream,
@@ -526,11 +616,17 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
                                   SVN_REPOS_DUMPFILE_TEXT_CONTENT_LENGTH,
                                   APR_HASH_KEY_STRING)))
         {
-          SVN_ERR (parse_text_block (stream, 
+          const char *delta = apr_hash_get (headers,
+                                            SVN_REPOS_DUMPFILE_TEXT_DELTA,
+                                            APR_HASH_KEY_STRING);
+          svn_boolean_t is_delta = (delta && strcmp (delta, "true") == 0);
+
+          SVN_ERR (parse_text_block (stream,
                                      svn__atoui64 (valstr),
+                                     is_delta,
                                      parse_fns,
                                      found_node ? node_baton : rev_baton,
-                                     buffer, 
+                                     buffer,
                                      buflen,
                                      found_node ? nodepool : revpool));
         }
@@ -558,6 +654,19 @@ svn_repos_parse_dumpstream (svn_stream_t *stream,
 }
 
 
+svn_error_t *
+svn_repos_parse_dumpstream (svn_stream_t *stream,
+                            const svn_repos_parser_fns_t *parse_fns,
+                            void *parse_baton,
+                            svn_cancel_func_t cancel_func,
+                            void *cancel_baton,
+                            apr_pool_t *pool)
+{
+  svn_repos_parser_fns2_t *fns2 = fns2_from_fns (parse_fns, pool);
+
+  return svn_repos_parse_dumpstream2 (stream, fns2, parse_baton,
+                                      cancel_func, cancel_baton, pool);
+}
 
 /*----------------------------------------------------------------------*/
 
@@ -665,6 +774,7 @@ new_revision_record (void **revision_baton,
   struct parse_baton *pb = parse_baton;
   struct revision_baton *rb;
   svn_revnum_t head_rev;
+  svn_revnum_t *old_rev, *new_rev;
 
   rb = make_revision_baton (headers, pb, pool);
   SVN_ERR (svn_fs_youngest_rev (&head_rev, pb->fs, pool));
@@ -672,6 +782,13 @@ new_revision_record (void **revision_baton,
   /* Calculate the revision 'offset' for finding copyfrom sources.
      It might be positive or negative. */
   rb->rev_offset = (rb->rev) - (head_rev + 1);
+ 
+  /* Store the new revision for finding copyfrom sources. */
+  old_rev = apr_palloc (pb->pool, sizeof(svn_revnum_t) * 2);
+  new_rev = old_rev + 1;
+  *old_rev = rb->rev;
+  *new_rev = head_rev + 1;
+  apr_hash_set (pb->rev_map, old_rev, sizeof(svn_revnum_t), new_rev);
 
   if (rb->rev > 0)
     {
@@ -680,10 +797,9 @@ new_revision_record (void **revision_baton,
       SVN_ERR (svn_fs_txn_root (&(rb->txn_root), rb->txn, pool));
       
       if (pb->outstream)
-        svn_stream_printf (pb->outstream, pool,
-                           "<<< Started new transaction, based on "
-                           "original revision %"
-                           SVN_REVNUM_T_FMT "\n", rb->rev);
+        SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                    _("<<< Started new transaction, based on "
+                                      "original revision %ld\n"), rb->rev));
     }
 
   /* If we're parsing revision 0, only the revision are (possibly)
@@ -719,11 +835,15 @@ maybe_add_with_history (struct node_baton *nb,
       /* Hunt down the source revision in this fs. */
       svn_fs_root_t *copy_root;
       svn_revnum_t src_rev = nb->copyfrom_rev - rb->rev_offset;
+      svn_revnum_t *src_rev_from_map;
+      if ((src_rev_from_map = apr_hash_get (pb->rev_map, &nb->copyfrom_rev,
+                                            sizeof(nb->copyfrom_rev))))
+        src_rev = *src_rev_from_map;
 
       if (! SVN_IS_VALID_REVNUM(src_rev))
         return svn_error_createf (SVN_ERR_FS_NO_SUCH_REVISION, NULL,
-                                  "Relative source revision %" SVN_REVNUM_T_FMT
-                                  " is not available in current repository",
+                                  _("Relative source revision %ld is not"
+                                    " available in current repository"),
                                   src_rev);
 
       SVN_ERR (svn_fs_revision_root (&copy_root, pb->fs, src_rev, pool));
@@ -733,7 +853,7 @@ maybe_add_with_history (struct node_baton *nb,
       if (pb->outstream)
         {
           apr_size_t len = 9;
-          svn_stream_write (pb->outstream, "COPIED...", &len);
+          SVN_ERR (svn_stream_write (pb->outstream, "COPIED...", &len));
         }
     }
 
@@ -770,30 +890,40 @@ new_node_record (void **node_baton,
 {
   struct revision_baton *rb = revision_baton;
   struct parse_baton *pb = rb->pb;
-  struct node_baton *nb = make_node_baton (headers, rb, pool);
+  struct node_baton *nb;
+  
+  if (rb->rev == 0)
+    return svn_error_create (SVN_ERR_STREAM_MALFORMED_DATA, NULL,
+                             "Malformed dumpstream: "
+                             "Revision 0 must not contain node records");
+
+  nb = make_node_baton (headers, rb, pool);
 
   switch (nb->action)
     {
     case svn_node_action_change:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * editing path : %s ...", nb->path);
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * editing path : %s ...",
+                                      nb->path));
         break;
       }
     case svn_node_action_delete:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * deleting path : %s ...", nb->path);
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * deleting path : %s ...",
+                                      nb->path));
         SVN_ERR (svn_fs_delete (rb->txn_root, nb->path, pool));
         break;
       }
     case svn_node_action_add:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * adding path : %s ...", nb->path);
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * adding path : %s ...",
+                                      nb->path));
 
         SVN_ERR (maybe_add_with_history (nb, rb, pool));
         break;
@@ -801,8 +931,9 @@ new_node_record (void **node_baton,
     case svn_node_action_replace:
       {
         if (pb->outstream)
-          svn_stream_printf (pb->outstream, pool,
-                             "     * replacing path : %s ...", nb->path);
+          SVN_ERR (svn_stream_printf (pb->outstream, pool,
+                                      "     * replacing path : %s ...",
+                                      nb->path));
 
         SVN_ERR (svn_fs_delete (rb->txn_root, nb->path, pool));
 
@@ -868,6 +999,20 @@ set_node_property (void *baton,
 }
 
 
+static svn_error_t *
+delete_node_property (void *baton,
+                      const char *name)
+{
+  struct node_baton *nb = baton;
+  struct revision_baton *rb = nb->rb;
+
+  SVN_ERR (svn_fs_change_node_prop (rb->txn_root, nb->path,
+                                    name, NULL, nb->pool));
+  
+  return SVN_NO_ERROR;
+}
+
+
 static svn_error_t *
 remove_node_props (void *baton)
 {
@@ -896,6 +1041,21 @@ remove_node_props (void *baton)
 }
 
 
+static svn_error_t *
+apply_textdelta (svn_txdelta_window_handler_t *handler,
+                 void **handler_baton,
+                 void *node_baton)
+{
+  struct node_baton *nb = node_baton;
+  struct revision_baton *rb = nb->rb;
+
+  return svn_fs_apply_textdelta (handler, handler_baton,
+                                 rb->txn_root, nb->path,
+                                 NULL, nb->md5_checksum,
+                                 nb->pool);
+}
+
+
 static svn_error_t *
 set_fulltext (svn_stream_t **stream,
               void *node_baton)
@@ -920,7 +1080,7 @@ close_node (void *baton)
   if (pb->outstream)
     {
       apr_size_t len = 7;
-      svn_stream_write (pb->outstream, " done.\n", &len);
+      SVN_ERR (svn_stream_write (pb->outstream, " done.\n", &len));
     }
   
   return SVN_NO_ERROR;
@@ -965,16 +1125,16 @@ close_revision (void *baton)
     {
       if (new_rev == rb->rev)
         {
-          svn_stream_printf (pb->outstream, rb->pool,
-                             "\n------- Committed revision %" SVN_REVNUM_T_FMT
-                             " >>>\n\n", new_rev);
+          SVN_ERR (svn_stream_printf (pb->outstream, rb->pool,
+                                      _("\n------- Committed revision %ld"
+                                        " >>>\n\n"), new_rev));
         }
       else
         {
-          svn_stream_printf (pb->outstream, rb->pool,
-                             "\n------- Committed new rev %" SVN_REVNUM_T_FMT
-                             " (loaded from original rev %" SVN_REVNUM_T_FMT
-                             ") >>>\n\n", new_rev, rb->rev);
+          SVN_ERR (svn_stream_printf (pb->outstream, rb->pool,
+                                      _("\n------- Committed new rev %ld"
+                                        " (loaded from original rev %ld"
+                                        ") >>>\n\n"), new_rev, rb->rev));
         }
     }
 
@@ -987,6 +1147,33 @@ close_revision (void *baton)
 /** The public routines **/
 
 
+svn_error_t *
+svn_repos_get_fs_build_parser2 (const svn_repos_parser_fns2_t **callbacks,
+                                void **parse_baton,
+                                svn_repos_t *repos,
+                                svn_boolean_t use_history,
+                                enum svn_repos_load_uuid uuid_action,
+                                svn_stream_t *outstream,
+                                const char *parent_dir,
+                                apr_pool_t *pool)
+{
+  const svn_repos_parser_fns_t *fns;
+  svn_repos_parser_fns2_t *parser;
+
+  /* Fetch the old-style vtable and baton, convert the vtable to a
+   * new-style vtable, and set the new callbacks. */
+  SVN_ERR (svn_repos_get_fs_build_parser (&fns, parse_baton, repos,
+                                          use_history, uuid_action, outstream,
+                                          parent_dir, pool));
+  parser = fns2_from_fns (fns, pool);
+  parser->delete_node_property = delete_node_property;
+  parser->apply_textdelta = apply_textdelta;
+  *callbacks = parser;
+  return SVN_NO_ERROR;
+}
+
+
+
 svn_error_t *
 svn_repos_get_fs_build_parser (const svn_repos_parser_fns_t **parser_callbacks,
                                void **parse_baton,
@@ -1016,6 +1203,8 @@ svn_repos_get_fs_build_parser (const svn_repos_parser_fns_t **parser_callbacks,
   pb->outstream = outstream;
   pb->uuid_action = uuid_action;
   pb->parent_dir = parent_dir;
+  pb->pool = pool;
+  pb->rev_map = apr_hash_make (pool);
 
   *parser_callbacks = parser;
   *parse_baton = pb;
@@ -1034,21 +1223,21 @@ svn_repos_load_fs (svn_repos_t *repos,
                    void *cancel_baton,
                    apr_pool_t *pool)
 {
-  const svn_repos_parser_fns_t *parser;
+  const svn_repos_parser_fns2_t *parser;
   void *parse_baton;
   
   /* This is really simple. */  
 
-  SVN_ERR (svn_repos_get_fs_build_parser (&parser, &parse_baton,
-                                          repos,
-                                          TRUE, /* look for copyfrom revs */
-                                          uuid_action,
-                                          feedback_stream,
-                                          parent_dir,
-                                          pool));
+  SVN_ERR (svn_repos_get_fs_build_parser2 (&parser, &parse_baton,
+                                           repos,
+                                           TRUE, /* look for copyfrom revs */
+                                           uuid_action,
+                                           feedback_stream,
+                                           parent_dir,
+                                           pool));
 
-  SVN_ERR (svn_repos_parse_dumpstream (dumpstream, parser, parse_baton,
-                                       cancel_func, cancel_baton, pool));
+  SVN_ERR (svn_repos_parse_dumpstream2 (dumpstream, parser, parse_baton,
+                                        cancel_func, cancel_baton, pool));
 
   return SVN_NO_ERROR;
 }
