@@ -647,8 +647,8 @@ svn_config_set_bool (svn_config_t *cfg,
 
 
 int
-svn_config__enumerate_sections (svn_config_t *cfg,
-                               svn_config__section_enumerator_t callback,
+svn_config_enumerate_sections (svn_config_t *cfg,
+                               svn_config_section_enumerator_t callback,
                                void *baton)
 {
   apr_hash_index_t *sec_ndx;
