@@ -314,8 +314,8 @@ svn_config__user_config_path (const char *config_dir,
     apr_status_t apr_err;
     apr_uid_t uid;
     apr_gid_t gid;
-    char *username;
-    char *homedir;
+    char *username, *homedir;
+    const char *homedir_utf8;
 
     apr_err = apr_uid_current (&uid, &gid, pool);
     if (apr_err)
@@ -328,9 +328,11 @@ svn_config__user_config_path (const char *config_dir,
     apr_err = apr_uid_homepath_get (&homedir, username, pool);
     if (apr_err)
       return SVN_NO_ERROR;
-    
+
+    SVN_ERR (svn_utf_cstring_to_utf8 (&homedir_utf8, homedir, pool));
+
     *path_p = svn_path_join_many (pool,
-                                  svn_path_canonicalize (homedir, pool),
+                                  svn_path_canonicalize (homedir_utf8, pool),
                                   SVN_CONFIG__USR_DIRECTORY, fname, NULL);
     
   }
@@ -480,7 +482,6 @@ ensure_auth_dirs (const char *path,
                   apr_pool_t *pool)
 {
   svn_node_kind_t kind;
-  apr_status_t apr_err;
   const char *auth_dir, *auth_subdir;
   svn_error_t *err;
 
@@ -491,11 +492,15 @@ ensure_auth_dirs (const char *path,
     {
       svn_error_clear (err);
       /* 'chmod 700' permissions: */
-      apr_err = apr_dir_make (auth_dir,
-                              (APR_UREAD | APR_UWRITE | APR_UEXECUTE),
-                              pool);
-      if (apr_err)
-        return;
+      err = svn_io_dir_make (auth_dir,
+                             (APR_UREAD | APR_UWRITE | APR_UEXECUTE),
+                             pool);
+      if (err)
+        {
+          /* Don't try making subdirs if we can't make the top-level dir. */
+          svn_error_clear (err);
+          return;
+        }
     }
 
   /* If a provider exists that wants to store credentials in
@@ -507,7 +512,7 @@ ensure_auth_dirs (const char *path,
   if (err || kind == svn_node_none)
     {
       svn_error_clear (err);
-      apr_err = apr_dir_make (auth_subdir, APR_OS_DEFAULT, pool);
+      svn_error_clear (svn_io_dir_make (auth_subdir, APR_OS_DEFAULT, pool));
     }
       
   auth_subdir = svn_path_join_many (pool, auth_dir,
@@ -516,7 +521,7 @@ ensure_auth_dirs (const char *path,
   if (err || kind == svn_node_none)
     {
       svn_error_clear (err);
-      apr_err = apr_dir_make (auth_subdir, APR_OS_DEFAULT, pool);
+      svn_error_clear (svn_io_dir_make (auth_subdir, APR_OS_DEFAULT, pool));
     }
 
   auth_subdir = svn_path_join_many (pool, auth_dir,
@@ -525,7 +530,7 @@ ensure_auth_dirs (const char *path,
   if (err || kind == svn_node_none)
     {
       svn_error_clear (err);
-      apr_err = apr_dir_make (auth_subdir, APR_OS_DEFAULT, pool);
+      svn_error_clear (svn_io_dir_make (auth_subdir, APR_OS_DEFAULT, pool));
     }
 }
 
@@ -535,7 +540,6 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
 {
   const char *path;
   svn_node_kind_t kind;
-  apr_status_t apr_err;
   svn_error_t *err;
 
   /* Ensure that the user-specific config directory exists.  */
@@ -547,9 +551,13 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
   SVN_ERR (svn_io_check_path (path, &kind, pool));
   if (kind == svn_node_none)
     {
-      apr_err = apr_dir_make (path, APR_OS_DEFAULT, pool);
-      if (apr_err)
-        return SVN_NO_ERROR;
+      err = svn_io_dir_make (path, APR_OS_DEFAULT, pool);
+      if (err)
+        {
+          /* Don't throw an error, but don't continue. */
+          svn_error_clear (err);
+          return SVN_NO_ERROR;
+        }
     }
   else
     {
@@ -802,17 +810,19 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
    APR_EOL_STR
    APR_EOL_STR;
 
-      apr_err = apr_file_open (&f, path,
-                               (APR_WRITE | APR_CREATE | APR_EXCL),
-                               APR_OS_DEFAULT,
-                               pool);
+      err = svn_io_file_open (&f, path,
+                              (APR_WRITE | APR_CREATE | APR_EXCL),
+                              APR_OS_DEFAULT,
+                              pool);
 
-      if (! apr_err)
+      if (! err)
         {
           SVN_ERR (svn_io_file_write_full (f, contents, 
                                            strlen (contents), NULL, pool));
           SVN_ERR (svn_io_file_close (f, pool));
         }
+
+      svn_error_clear (err);
     }
 
   /** Ensure that the `servers' file exists. **/
@@ -858,7 +868,8 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
         APR_EOL_STR
         "###   ssl-trust-default-ca       Trust the system 'default' CAs" 
         APR_EOL_STR
-        "###   ssl-client-cert-file       PKCS#12 format client certificate file"
+        "###   ssl-client-cert-file       PKCS#12 format client "
+        "certificate file"
         APR_EOL_STR
         "###   ssl-client-cert-password   Client Key password, if needed."
         APR_EOL_STR
@@ -899,7 +910,7 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
         APR_EOL_STR
         "# [group1]"
         APR_EOL_STR
-        "# http-proxy-host = proxy1.some-domain-name.com"
+"# http-proxy-host = proxy1.some-domain-name.com"
         APR_EOL_STR
         "# http-proxy-port = 80"
         APR_EOL_STR
@@ -969,7 +980,7 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
         APR_EOL_STR
         "# http-proxy-password = defaultpassword"
         APR_EOL_STR
-        "# http-compression = yes"
+        "# http-compression = no"
         APR_EOL_STR
         "# No http-timeout, so just use the builtin default."
         APR_EOL_STR
@@ -978,17 +989,19 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
         "# ssl-authority-files = /path/to/CAcert.pem;/path/to/CAcert2.pem"
         APR_EOL_STR;
 
-      apr_err = apr_file_open (&f, path,
-                               (APR_WRITE | APR_CREATE | APR_EXCL),
-                               APR_OS_DEFAULT,
-                               pool);
+      err = svn_io_file_open (&f, path,
+                              (APR_WRITE | APR_CREATE | APR_EXCL),
+                              APR_OS_DEFAULT,
+                              pool);
 
-      if (! apr_err)
+      if (! err)
         {
           SVN_ERR (svn_io_file_write_full (f, contents, 
                                            strlen (contents), NULL, pool));
           SVN_ERR (svn_io_file_close (f, pool));
         }
+
+      svn_error_clear (err);
     }
 
   /** Ensure that the `config' file exists. **/
@@ -1017,7 +1030,21 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
         APR_EOL_STR
         "### Section for authentication and authorization customizations."
         APR_EOL_STR
-        "### Set store-auth-creds to 'no' to avoid storing your subversion"
+        "# [auth]"
+        APR_EOL_STR
+        "### Set store-passwords to 'no' to avoid storing passwords in the"
+        APR_EOL_STR
+        "### auth/ area of your config directory.  It defaults to 'yes'."
+        APR_EOL_STR
+        "### Note that this option only prevents saving of *new* passwords;"
+        APR_EOL_STR
+        "### it doesn't invalidate existing passwords.  (To do that, remove"
+        APR_EOL_STR
+        "### the cache files by hand as described in the Subversion book.)"
+        APR_EOL_STR
+        "# store-passwords = no"
+        APR_EOL_STR
+        "### Set store-auth-creds to 'no' to avoid storing any subversion"
         APR_EOL_STR
         "### credentials in the auth/ area of your config directory."
         APR_EOL_STR
@@ -1027,8 +1054,6 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
         APR_EOL_STR
         "### caches.  (To do that, remove the cache files by hand.)"
         APR_EOL_STR
-        "# [auth]"
-        APR_EOL_STR
         "# store-auth-creds = no"
         APR_EOL_STR
         APR_EOL_STR
@@ -1187,17 +1212,19 @@ svn_config_ensure (const char *config_dir, apr_pool_t *pool)
         APR_EOL_STR
         APR_EOL_STR;
         
-      apr_err = apr_file_open (&f, path,
-                               (APR_WRITE | APR_CREATE | APR_EXCL),
-                               APR_OS_DEFAULT,
-                               pool);
+      err = svn_io_file_open (&f, path,
+                              (APR_WRITE | APR_CREATE | APR_EXCL),
+                              APR_OS_DEFAULT,
+                              pool);
 
-      if (! apr_err)
+      if (! err)
         {
           SVN_ERR (svn_io_file_write_full (f, contents, 
                                            strlen (contents), NULL, pool));
           SVN_ERR (svn_io_file_close (f, pool));
         }
+
+      svn_error_clear (err);
     }
 
   return SVN_NO_ERROR;
