@@ -599,58 +599,46 @@ def status_add_deleted_directory(sbox):
   expected_status.tweak('', 'iota', wc_rev=2)
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
-#----------------------------------------------------------------------
-# Regression test for issue #1775:
-# Should be able to revert a file with no properties i.e. no prop-base 
-def revert_replaced_file_without_props(sbox):
-  "revert a replaced file with no properties"
 
-  sbox.build()
+#----------------------------------------------------------------------
+# Regression test for issue #939:
+# Recursive 'svn add' should still traverse already-versioned dirs.
+def add_recursive_already_versioned(sbox):
+  "'svn add' should traverse already-versioned dirs"
+
   wc_dir = sbox.wc_dir
 
-  file1_path = os.path.join(wc_dir, 'file1')
+  if svntest.actions.make_repo_and_wc(sbox):
+    return 1
 
-  # Add a new file, file1, that has no prop-base
-  svntest.main.file_append(file1_path, "This is the file 'file1' revision 2.")
-  svntest.actions.run_and_verify_svn(None, None, [], 'add', file1_path)
-
-  # commit file1
-  expected_output = svntest.wc.State(wc_dir, {
-    'file1' : Item(verb='Adding')
-    })
-
-  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
-  expected_status.tweak(wc_rev=1)
+  # Create some files, then schedule them for addition
+  delta_path = os.path.join(wc_dir, 'delta')
+  zeta_path = os.path.join(wc_dir, 'A', 'B', 'zeta')
+  epsilon_path = os.path.join(wc_dir, 'A', 'D', 'G', 'epsilon')
+  
+  svntest.main.file_append(delta_path, "This is the file 'delta'.")
+  svntest.main.file_append(zeta_path, "This is the file 'zeta'.")
+  svntest.main.file_append(epsilon_path, "This is the file 'epsilon'.")
+  
+  saved_wd = os.getcwd()
+  try:
+    os.chdir(wc_dir)
+    svntest.main.run_svn(None, 'add', '--force', '.')
+  finally:
+    os.chdir(saved_wd)
+  
+  # Make sure the adds show up as such in status
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.add({
-    'file1' : Item(status='  ', wc_rev=2, repos_rev=2),
+    'delta' : Item(status='A ', wc_rev=0, repos_rev=1),
+    'A/B/zeta' : Item(status='A ', wc_rev=0, repos_rev=1),
+    'A/D/G/epsilon' : Item(status='A ', wc_rev=0, repos_rev=1),
     })
 
-  svntest.actions.run_and_verify_commit (wc_dir, expected_output,
-                                         expected_status, None, None,
-                                         None, None, None, wc_dir)
+  return svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
-  # delete file1 
-  svntest.actions.run_and_verify_svn(None, None, [], 'rm', file1_path)
 
-  # test that file1 is scheduled for deletion.
-  expected_status.tweak('file1', status='D ')
-  svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
-  # recreate and add file1 
-  svntest.main.file_append(file1_path, "This is the file 'file1' revision 3.")
-  svntest.actions.run_and_verify_svn(None, None, [], 'add', file1_path)
-
-  # Test to see if file1 is schedule for replacement 
-  expected_status.tweak('file1', status='R ')
-  svntest.actions.run_and_verify_status(wc_dir, expected_status)
-
-  # revert file1
-  svntest.actions.run_and_verify_svn(None, ["Reverted '" + file1_path + "'\n"],
-                                     None, 'revert', file1_path)
-
-    # test that file1 really was reverted
-  expected_status.tweak('file1', status='  ', wc_rev=2, repos_rev=2)
-  svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
 ########################################################################
 # Run the tests
@@ -680,7 +668,7 @@ test_list = [ None,
               delete_missing,
               revert_inside_newly_added_dir,
               status_add_deleted_directory,
-              revert_replaced_file_without_props,
+              add_recursive_already_versioned,
              ]
 
 if __name__ == '__main__':
